/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
export function bindFuncObj(funcObj = {}, context) {
  return Object.keys(funcObj).reduce((obj, fnName) => {
    obj[fnName] = funcObj[fnName].bind(context);
    return obj;
  }, {});
}

/**
 * 用于处理自定义组件props传参结构，对系统变量进行保留
 */
export function resolveComponentProps(props, isPlainProps) {
  const { staticResourceAttribute } = props;
  staticResourceAttribute &&
    staticResourceAttribute.map((property) => {
      if (props.data && props.data[property]) {
        props.data[property] = getStaticResourceAttribute(props.data[property]);
      }
    });
  if (!isPlainProps) {
    return {
      ...props,
    };
  }
  const { data = {}, events = [], ...restProps } = props;
  const customProps = { ...data };
  const builtinProps = [
    // react 保留字
    'ref',
    'key',
    'dangerouslySetInnerHTML',
    'className',
    'htmlFor',
    'style',
    'contentEditable',
    // lowcode 保留字
    'events',
    'children',
    '_parentId',
    '_visible',
    'classList',
    'widgetType',
    'getWidgetsByType',
    'getDom',
    'domRef',
    'extends',
    // 小程序保留字
    'id',
    'class',
    'hidden',
    'slot',
  ];
  // delete builtin props
  builtinProps.map((prop) => delete customProps[prop]);
  // 选区events处理
  const { _selectableBlockEvents, ...restData } = data;
  const { emit: _selectableBlockEmit } = _selectableBlockEvents;
  _selectableBlockEvents.events = new Proxy(
    _selectableBlockEvents.events.reduce((events, item) => {
      const propName = item;
      events[propName] = (e) => _selectableBlockEmit(propName, e);
      return events;
    }, {}),
    {
      get(obj, prop) {
        return prop in obj ? obj[prop] : (e) => _selectableBlockEmit(prop, e);
      },
    },
  );
  return {
    ...restData,
    ...restProps,
    _selectableBlockEvents,
    events: new Proxy(
      events.reduce((events, item) => {
        const propName = item;
        events[propName] = (e) => restProps.emit(propName, e);
        return events;
      }, {}),
      {
        get(obj, prop) {
          return prop in obj ? obj[prop] : (e) => restProps.emit(prop, e);
        },
      },
    ),
  };
}

function getStaticResourceAttribute(staticUrl) {
  if (/^\//.test(staticUrl)) {
    const { domain = '' } = window.app || {};
    const url = `https://${domain}${staticUrl}`;
    return url;
  }
  return staticUrl;
}
